/*
 * Project name:
     TouchPanel (Demo for working with TouchPanel Controller)
 * Copyright:
     (c) mikroElektronika, 2009.
 * Revision History:
     20080930:
       - initial release;
       - 20090818 - Slavisa Zlatanovic;
 * Description:
     This code works with TouchPanel and GLCD. Two digital output and
     two analog input signals are used for communication with TouchPanel.
 * Test configuration:
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/en/tools/easyavr6/
     Oscillator:      External Clock 08.0000 MHz
     Ext. Modules:    GLCD 128x64, Touch Panel
                      http://www.mikroe.com/en/tools/components/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/en/compilers/mikroc/avr/
 * Notes:
     - Turn on GLCD backlight (SW10.7) and TouchPanel Contoller switches (SW8.5, SW8.6, SW8.7 and SW8.8).
     - Turn off diodes connected to ADC pins.
*/  

// Glcd module connections
char GLCD_DataPort at PORTC;
char GLCD_DataPort_Direction at DDRC;

sbit GLCD_CS1 at PORTD.B2;
sbit GLCD_CS2 at PORTD.B3;
sbit GLCD_RS  at PORTD.B4;
sbit GLCD_RW  at PORTD.B5;
sbit GLCD_EN  at PORTD.B6;
sbit GLCD_RST at PORTD.B7;
sbit GLCD_CS1_Direction at DDRD.B2;
sbit GLCD_CS2_Direction at DDRD.B3;
sbit GLCD_RS_Direction  at DDRD.B4;
sbit GLCD_RW_Direction  at DDRD.B5;
sbit GLCD_EN_Direction  at DDRD.B6;
sbit GLCD_RST_Direction at DDRD.B7;
// End Glcd module connections


// Touch Panel module connections
sbit DRIVE_A at PORTA.B2;
sbit DRIVE_B at PORTA.B3;
sbit DRIVE_A_Direction at DDRA.B2;
sbit DRIVE_B_Direction at DDRA.B3;
const char READ_X_CHANNEL = 0;                             // READ X line is connected to analog channel 0
const char READ_Y_CHANNEL = 1;                             // READ Y line is connected to analog channel 1
// End Touch Panel module connections

bit          write_erase;
char         pen_size;
unsigned int x_coord, y_coord, x_coord_old, y_coord_old, x_coord_diff, y_coord_diff;
long         x_coord128, y_coord64;                        // Scaled x-y position
int          cal_x_min, cal_y_min, cal_x_max, cal_y_max;   // Calibration constants
char write_msg[] = "WRITE";                                // GLCD menu messages
char clear_msg[] = "CLEAR";
char erase_msg[] = "ERASE";
const unsigned int ADC_THRESHOLD = 900;                    // Threshold value for press detecting

// Returns 1 if TouchPanel is pressed, 0 otherwise
char PressDetect() {
  unsigned adc_rd;
  char result;

  // PRESS detecting
  DRIVE_A = 0;                                             // DRIVEA = 0 (LEFT drive off, RIGHT drive off, TOP drive on)
  DRIVE_B = 0;                                             // DRIVEB = 0 (BOTTOM drive off)
  Delay_ms(5);

  adc_rd = ADC_Read(READ_Y_CHANNEL);                       // READ-Y
  result = (adc_rd > ADC_THRESHOLD);                       // If logical one is detectet

  // Debouncing, repeat detecting after 2ms
  Delay_ms(2);
  adc_rd = ADC_Read(READ_Y_CHANNEL);                       // READ-Y
  result = result & (adc_rd > ADC_THRESHOLD);
  return result;
}

unsigned int GetX() {
  unsigned int result;
  // Reading X
  DRIVE_A = 1;                                             // DRIVEA = 1 (LEFT drive on, RIGHT drive on, TOP drive off)
  DRIVE_B = 0;                                             // DRIVEB = 0 (BOTTOM drive off)

  Delay_ms(5);
  result = ADC_Read(READ_X_CHANNEL);                       // READ-X (BOTTOM)
  return result;
}

unsigned int GetY() {
  unsigned int result;
  // Reading Y
  DRIVE_A = 0;                                             // DRIVEA = 0 (LEFT drive off, RIGHT drive off, TOP drive on)
  DRIVE_B = 1;                                             // DRIVEB = 1 (BOTTOM drive on)

  Delay_ms(5);
  result = ADC_Read(READ_Y_CHANNEL);                       // READ-X (LEFT)
  return result;
}

void Calibrate() {

  Glcd_Dot(0,63,1);
  Glcd_Write_Text("TOUCH BOTTOM LEFT",12,3,1);

  while (!PressDetect());

  // Get calibration constants (reading and compensating TouchPanel nonlinearity)
  cal_x_min = GetX() - 10;
  cal_y_min = GetY() - 10;
  Delay_ms(1000);

  Glcd_Fill(0);
  Glcd_Dot(127,0,1);
  Glcd_Write_Text("TOUCH UPPER RIGHT",12,4,1);
  while (!PressDetect()) ;

  // Get calibration constants (reading and compensating TouchPanel nonlinearity)
  cal_x_max = GetX() + 5;
  cal_y_max = GetY() + 5;
  Delay_ms(1000);
}

void Initialize() {
  DRIVE_A_Direction = 1;                                   // Set DRIVE_A pin as output
  DRIVE_B_Direction = 1;                                   // Set DRIVE_B pin as output
  
  Glcd_Init();                                             // Initialize GLCD
}

void main() {

  Initialize();

  Glcd_Fill(0);                                            // Clear GLCD
  Glcd_Set_Font(font5x7, 5, 7, 32);                        // Choose font, see __Lib_GLCDFonts.c in Uses folder
  Glcd_Write_Text("CALIBRATION", 30, 2, 1);
  Delay_ms(1500);

  Glcd_Fill(0);
  Calibrate();
  Glcd_Fill(0);

  Glcd_Write_Text("WRITE ON SCREEN", 20, 5, 1) ;
  Delay_ms(1000);
  Glcd_Fill(0);

  Glcd_Fill(0);
  Glcd_V_Line(0,7,0,1);
  Glcd_Write_Text(clear_msg,1,0,0);
  Glcd_V_Line(0,7,97,1);
  Glcd_Write_Text(erase_msg,98,0,0);

  // Pen Menu:
  Glcd_Rectangle(41,0,52,9,1);
  Glcd_Box(45,3,48,6,1);
  Glcd_Rectangle(63,0,70,7,1);
  Glcd_Box(66,3,67,4,1);
  Glcd_Rectangle(80,0,86,6,1);
  Glcd_Dot(83,3,1);

  write_erase = 1;
  pen_size = 1;
  x_coord_old = 0;
  y_coord_old = 0;
  
  while (1) {

    if (PressDetect()) {
      // After a PRESS is detected read X-Y and convert it to 128x64 space
      x_coord = GetX() - cal_x_min;
      y_coord = GetY() - cal_y_min;
      
      
      // When lifting pen from the touchpanel surface GetX and GetY readings
      // (after correct PressDetect reading) may be incorrect
      x_coord_diff = abs(x_coord - x_coord_old);           // Compare with old values
      y_coord_diff = abs(y_coord - y_coord_old);
      x_coord_old = x_coord;                               // Save old values
      y_coord_old = y_coord;
      
      if ( (x_coord_diff>50) || (y_coord_diff>50) )        // If difference is too big then ignore the reading
        continue;

      x_coord128 = (x_coord * 128l) / (cal_x_max - cal_x_min);
      y_coord64 = (64 -(y_coord *64) / (cal_y_max - cal_y_min));

      if ((x_coord128 < 0) || (x_coord128 > 127))
        continue;
      if ((y_coord64 < 0) || (y_coord64 > 63))
        continue;


      // If clear is pressed
      if ((x_coord128 < 31) && (y_coord64 < 8)) {

        Glcd_Fill(0);

        // Pen Menu:
        Glcd_Rectangle(41,0,52,9,1);
        Glcd_Box(45,3,48,6,1);
        Glcd_Rectangle(63,0,70,7,1);
        Glcd_Box(66,3,67,4,1);
        Glcd_Rectangle(80,0,86,6,1);
        Glcd_Dot(83,3,1);

        Glcd_V_Line(0,7,0,1);
        Glcd_Write_Text(clear_msg,1,0,0);
        Glcd_V_Line(0,7,97,1);
        if (write_erase)
          Glcd_Write_Text(erase_msg,98,0,0);
        else
          Glcd_Write_Text(write_msg,98,0,0);
        }

      // If write/erase is pressed
      if ((x_coord128 > 96) && (y_coord64 < 8)) {
        if (write_erase) {
          write_erase = 0;
          Glcd_Write_Text(write_msg,98,0,0);
          Delay_ms(500);
          }
        else {
          write_erase = 1;
          Glcd_Write_Text(erase_msg,98,0,0);
          Delay_ms(500);
          }
        }

      // If pen size is selected
      if ((x_coord128 >= 41) && (x_coord128 <= 52) && (y_coord64 <= 9))
        pen_size = 3;

      if ((x_coord128 >= 63) && (x_coord128 <= 70) && (y_coord64 <= 7))
        pen_size = 2;

      if ((x_coord128 >= 80) && (x_coord128 <= 86) && (y_coord64 <= 6))
        pen_size = 1;

      if (y_coord64 < 11)
        continue;

      switch (pen_size) {
        case 1 : Glcd_Dot(x_coord128, y_coord64, write_erase); break;
        case 2 : Glcd_Box(x_coord128, y_coord64, x_coord128 + 1, y_coord64 + 1, write_erase); break;
        case 3 : Glcd_Box(x_coord128-1, y_coord64-1, x_coord128 + 2, y_coord64 + 2, write_erase); break;
        }

    }
  }
}
